useDynLib(Delaporte, .registration = TRUE)

importFrom(stats,
           rpois,
           quantile,
           rgamma)
importFrom(parallel,
           detectCores)
export(ddelap,
       pdelap,
       qdelap,
       rdelap,
       MoMdelap,
       getDelapThreads,
       setDelapThreads)
